/****************************************************************************/
/*                         SQL Server 2016 Virtual Forum                    */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                      Database Creation Script                            */
/****************************************************************************/
set noexec off
go

use master
go

if convert(int,
			left(
				convert(nvarchar(128), serverproperty('ProductVersion')),
				charindex('.',convert(nvarchar(128), serverproperty('ProductVersion'))) - 1
			)
	) < 12 
begin
	raiserror('You should have SQL Server 2014 to execute this script',16,1) with nowait
	set noexec on
end
go

if convert(int, serverproperty('EngineEdition')) != 3 or charindex('X64',@@Version) = 0
begin
	raiserror('That script requires 64-Bit Enterprise Edition of SQL Server to run',16,1)
	set noexec on
end
go

if exists
(
	select * from sys.databases where name = 'InMemoryOLTP2016'
)
begin
	alter database InMemoryOLTP2016 set single_user with rollback immediate;
	drop database InMemoryOLTP2016;
end
go



declare
	@dataPath nvarchar(512) = convert(nvarchar(512),serverproperty('InstanceDefaultDataPath'))
	,@logPath nvarchar(512) = convert(nvarchar(512),serverproperty('InstanceDefaultLogPath'))

/*** REPLACE IF YOU WANT TO STORE IN-MEMORY OLTP FILES IN THE DIFFERENT PLACE ***/
declare
	@HKPath nvarchar(512) = @dataPath + 'InMemoryOLTP2016_HKData'
	
declare
	@SQL nvarchar(max)

select @SQL = 
N'create database [InMemoryOLTP2016] on 
primary (name=N''InMemoryOLTP2016'', filename=N''' + @dataPath + N'InMemoryOLTP2016.mdf'', size=10MB, filegrowth = 10MB),
filegroup [HKData] contains memory_optimized_data (name=N''InMemoryOLTP2016_HekatonData'', filename=N''' + @HKPath + N'''),
filegroup LOGDATA 
(name = N''LogData1'', filename=N''' + @dataPath + N'InMemoryOLTP2016LogData1.ndf''),
(name = N''LogData2'', filename=N''' + @dataPath + N'InMemoryOLTP2016LogData2.ndf''),
(name = N''LogData3'', filename=N''' + @dataPath + N'InMemoryOLTP2016LogData3.ndf''),
(name = N''LogData4'', filename=N''' + @dataPath + N'InMemoryOLTP2016LogData4.ndf''),
filegroup FG2015 
(name = N''FG2015_1'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2015_1.ndf''),
(name = N''FG2015_2'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2015_2.ndf''),
(name = N''FG2015_3'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2015_3.ndf''),
(name = N''FG2015_4'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2015_4.ndf''),
filegroup FG2016 
(name = N''FG2016_1'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2016_1.ndf''),
(name = N''FG2016_2'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2016_2.ndf''),
(name = N''FG2016_3'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2016_3.ndf''),
(name = N''FG2016_4'', filename=N''' + @dataPath + N'InMemoryOLTP2016FG2016_4.ndf'')
log on (name=N''InMemoryOLTP2016_log'', filename=N''' + @logPath + N'InMemoryOLTP2016.ldf'', size=256000KB, filegrowth = 256000KB);

alter database [InMemoryOLTP2016] set recovery simple;';

raiserror('Creating database InMemoryOLTP2016',0,1) with nowait;
raiserror('Data Path: %s',0,1,@dataPath) with nowait;
raiserror('Log Path: %s',0,1,@logPath) with nowait;
raiserror('In-Memory OLTP Folder: %s',0,1,@HKPath) with nowait;
raiserror('Statement:',0,1) with nowait;
raiserror(@sql,0,1) with nowait;

exec sp_executesql @sql;
go

use InMemoryOLTP2016
go

create table dbo.Numbers
( 
	Num int not null,
	constraint PK_Numbers
	primary key clustered(Num)
	on [Primary]
)
go

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as t1 cross join N1 as t2) -- 4 rows
,N3(C) as (select 0 from N2 as t1 cross join N2 as t2) -- 16 rows
,N4(C) as (select 0 from N3 as t1 cross join N3 as t2) -- 256 rows
,N5(C) as (select 0 from N4 as t1 cross join N4 as t2) -- 65,536 rows
,N6(C) as (select 0 from N5 as t1 cross join N3 as t2) -- 1,048,576 rows
,Ids(Id) as (select row_number() over (order by (select null)) from N6)
insert into dbo.Numbers(Num)
	select Id from ids;

