/****************************************************************************/
/*                         SQL Server 2016 Virtual Forum                    */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*      Performance of Memory-Optimized Table Variable in OLTP Workload     */
/****************************************************************************/
set nocount on
set xact_abort on
go

use InMemoryOLTP2016
go


if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'TestTempVars') drop proc dbo.TestTempVars; 
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'TestTempTables') drop proc dbo.TestTempTables; 
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'TestInMemTempTables') drop proc dbo.TestInMemTempTables; 
if exists(select * from sys.table_types t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'InMemTV') drop type dbo.InMemTV; 
go

create type dbo.InMemTV as table
(
	Id int not null
		primary key nonclustered hash
		with (bucket_count=512),
	Placeholder char(255)
)
with (memory_optimized=on)
go

create proc dbo.TestInMemTempTables(@Rows int)
as
	declare
		@ttTemp dbo.InMemTV
		,@Cnt int

	insert into @ttTemp	(Id)
		select Num from dbo.Numbers where Num <= @Rows;

	select @Cnt = count(*) from @ttTemp 
go


create proc dbo.TestTempTables(@Rows int)
as
	declare
		@Cnt int

	create table #TTTemp
	(
		Id int not null primary key,
		Placeholder char(255)
	)

	insert into #TTTemp	(Id)
		select Num from dbo.Numbers where Num <= @Rows;

	select @Cnt = count(*) from #TTTemp
go


create proc dbo.TestTempVars(@Rows int)
as
	declare
		@Cnt int

	declare 
		@ttTemp table
		(
			Id int not null primary key,
			Placeholder char(255)
		)

	insert into @ttTemp	(Id)
		select Num from dbo.Numbers where Num <= @Rows;

	select @Cnt = count(*) from @ttTemp 
go

declare
	@Result table
	(
		[Rows] int not null primary key,
		InMemTV int not null,
		TempTbl int not null,
		OnDiskTV int not null
	);

declare
	@PacketSize int = 16
	,@LoopCnt int = 5000
	,@I int = 0
	,@DT datetime
	,@InMemTV int
	,@TempTbl int
	,@OnDiskTV int 

while @PacketSize <= 256
begin
	select @DT = getDate(), @I = 0;
	while @I < @LoopCnt
	begin
		exec dbo.TestInMemTempTables @PacketSize;
		set @I += 1;
	end;
	select @InMemTV = datediff(millisecond,@DT,GetDate());
	
	select @I = 0, @DT = getdate();
	while @I < @LoopCnt
	begin
		exec dbo.TestTempTables @PacketSize;
		set @I += 1;
	end;
	select @TempTbl = datediff(millisecond,@DT,GetDate());
	
	select @I = 0, @DT = getdate();
	while @I < @LoopCnt
	begin
		exec dbo.TestTempVars @PacketSize;
		set @I += 1;
	end;
	select @OnDiskTV = datediff(millisecond,@DT,GetDate());

	insert into @Result([Rows],InMemTV,TempTbl,OnDiskTV)
	values(@PacketSize,@InMemTV,@TempTbl,@OnDiskTV);
	select @PacketSize *= 2;
end;

select * from @Result order by [Rows];
go




/* Checking Cardinality Estimations */
declare
	@InMemTV dbo.InMemTV;

declare 
	@ttTemp table
	(
		Id int not null primary key,
		Placeholder char(255)
	);

insert into @InMemTV(Id)
	select Num from dbo.Numbers where Num <= 256;

insert into @ttTemp(Id) 
	select Num from dbo.Numbers where Num <= 256;

select count(*) from @ttTemp   -- option (recompile);
select count(*) from @InMemTV  -- option (recompile);