/****************************************************************************/
/*                         SQL Server 2016 Virtual Forum                    */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                      Bicket_Count and performance                        */
/****************************************************************************/
set nocount on
set xact_abort on
go

use InMemoryOLTP2016
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where t.name = 'HashIndex_LowBucketCount' and s.name = 'dbo') drop table dbo.HashIndex_LowBucketCount;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where t.name = 'HashIndex_HighBucketCount' and s.name = 'dbo') drop table dbo.HashIndex_HighBucketCount;

create table dbo.HashIndex_LowBucketCount
(
	Id int not null
		constraint PK_HashIndex_LowBucketCount
		primary key nonclustered 
		hash with (bucket_count=1000),
	Value int not null
) 
with (memory_optimized=on, durability=schema_only);

create table dbo.HashIndex_HighBucketCount
(
	Id int not null
		constraint PK_HashIndex_HighBucketCount
		primary key nonclustered 
		hash with (bucket_count=1000000),
	Value int not null
) 
with (memory_optimized=on, durability=schema_only);
go

/* Insert Performance */
set statistics time on
insert into dbo.HashIndex_LowBucketCount(Id, Value)
	select Num, Num
	from dbo.Numbers with (tablock)
	where Num <= 400000;
	
insert into dbo.HashIndex_HighBucketCount(Id, Value)
	select Num, Num
	from dbo.Numbers with (tablock)
	where Num <= 400000;
set statistics time off
go

/* Select Performance */
set statistics time on
select avg(c.Cnt)
from dbo.Numbers n with (tablock)
	cross apply
	(
		select count(*) as Cnt
		from dbo.HashIndex_LowBucketCount h
		where h.Id = n.Num		
	) c
where n.Num <= 65536;
		
select avg(c.Cnt)
from dbo.Numbers n with (tablock)
	cross apply
	(
		select count(*) as Cnt
		from dbo.HashIndex_HighBucketCount h
		where h.Id = n.Num		
	) c
where n.Num <= 65536;
set statistics time off;
go

