/****************************************************************************/
/*                         SQL Server 2016 Virtual Forum                    */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*              Tiered Storage: Stored Procedures Creation                  */
/****************************************************************************/
set nocount on
set xact_abort on
go

/* This script requires SQL Server 2016. In SQL Server 2014 you can replace triggers
with the code on data access tier */

use InMemoryOLTP2016
go

if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'InsertPayments2016_06') drop proc dbo.InsertPayments2016_06;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'UpdatePayments2016_06') drop proc dbo.UpdatePayments2016_06;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'DeletePayments2016_06') drop proc dbo.DeletePayments2016_06;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'UpdatePayments2016') drop proc dbo.UpdatePayments2016;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'DeletePayments2016') drop proc dbo.DeletePayments2016;
go


create proc dbo.InsertPayments2016_06
(
	@PaymentDate datetime2(0) not null
	,@CustomerId int not null
	,@Amount money not null
	,@Status tinyint not null
	,@PaymentId uniqueidentifier output
)
with native_compilation, schemabinding, execute as owner
as
begin atomic with
(
    transaction isolation level = snapshot
    ,language = N'English'
)
	set @PaymentId = NewId();

	insert into dbo.Payments2016_06(PaymentDate,PaymentId,CustomerId,Amount,Status)
	values(@PaymentDate, @PaymentId,@CustomerId,@Amount,@Status);
end
go

create proc dbo.UpdatePayments2016_06
(
	@PaymentDate datetime2(0) not null
	,@PaymentId uniqueidentifier not null
	,@Amount money not null
	,@Status tinyint not null
)
with native_compilation, schemabinding, execute as owner
as
begin atomic with
(
    transaction isolation level = snapshot
    ,language = N'English'
)
	update dbo.Payments2016_06
	set Amount = @Amount, @Status = @Status
	where PaymentId = @PaymentId
end
go

create proc dbo.DeletePayments2016_06
(
	@PaymentDate datetime2(0) not null
	,@PaymentId uniqueidentifier not null
)
with native_compilation, schemabinding, execute as owner
as
begin atomic with
(
    transaction isolation level = snapshot
    ,language = N'English'
)
	delete from dbo.Payments2016_06
	where PaymentId = @PaymentId;
end
go

create proc dbo.UpdatePayments2016
(
	@PaymentDate datetime2(0) 
	,@PaymentId uniqueidentifier
	,@Amount money
	,@Status tinyint
)
as
	if @PaymentDate < '2016-05-01'
		update dbo.Payments2016
		set Amount = @Amount, @Status = @Status
		where PaymentId = @PaymentId and @PaymentDate = @PaymentDate
	else
		update dbo.Payments2016_05
		set Amount = @Amount, @Status = @Status
		where PaymentId = @PaymentId
go

create proc dbo.DeletePayments2016
(
	@PaymentDate datetime2(0) 
	,@PaymentId uniqueidentifier
)
as
	if @PaymentDate < '2016-05-01'
		delete from dbo.Payments2016
		where PaymentId = @PaymentId and @PaymentDate = @PaymentDate
	else 
		delete from dbo.Payments2016_05
		where PaymentId = @PaymentId
go
